% computes steady state 

function [ss,exitflag] = sstate(param);

pnames = { 	'alph  '   	% 1
            'bet   '    % 2
            'gamm  '  	% 3
            'kappa  '   % 4
            'omegaw'    % 5    
            'omegap'    % 6
            'etaw'      % 7
            'etap'      % 8
            'mu'        % 9 
            'phipi'     %10
            'phiy'      %11
            'sgc'       %12
            'sgi'       %13    
            'sb'        %14
            'taul'		%15
            'tauk'      %16
            'tauc'      %17
            'theta'     %18
            'gpsi'      %19
            's'         %20  s" in the investment adjustment cost function
            'chiw'      %21
            'chip'      %22
            'delt'      %23   
            'deltg'     %24
            'rhoa'		%25
            'rhob'      %26
            'rhor'      %27
            'rhoi'      %28
            'rhow'      %29
            'rhop'      %30
            'rhogc'		%31
            'rhogi'     %32
            'rhotk'     %33
            'rhotl'     %34
            'rhotc'     %35
            'rhoz'      %36
            'gammgc'	%37
            'gammgi'	%38
            'gammtk'	%39
            'gammtl'    %40
            'gammz'     %41
            'psitk'     %42
            'psitl'     %43
            'psigc'     %44
            'psiz'      %45
            'siga'      %46    
            'sigb'      %47
            'sigm'      %48
            'sigi'      %49
            'sigw'      %50
            'sigp'      %51
            'siggc'     %52
            'siggi'     %53
            'sigtk'     %54
            'sigtl'     %55
            'sigtc'     %56
            'sigz'      %57
            'alphg'     %58
};      


for j = 1:length(param)
    eval([char(pnames(j,:)) '=param(j);']);
end

phi = 1/s;
R = 1/bet;
rk = (1 - bet*(1 - delt))/(bet*(1 - tauk)*(1+tauc));
psi1 = rk*(1-tauk);

X(1) = 1;  %w
X(2) = 5;  %k/y
X(3) = 1;  %l/y
X(4) = .7; %c/y
X(5) = 1;  %y 

%Solve w, k/y, l/y, c/y, y jointly
options = optimset('Display','off','MaxIter',10000,'TolFun',1e-10);
[X,fval,exitflag] = fsolve(@ssNLEst,X,options,param,rk);
w = X(1);
ky = X(2);
ly = X(3);
cy = X(4);
y = X(5);

exitflag

str = (1 - R)*sb - sgc - sgi + (tauc/(1+tauc))*cy + taul*w*ly + tauk*rk*(ky); %transfer-output share
cnry = (1-taul)*w*ly + str;
cry = (cy - mu*cnry)/(1-mu);
cr = cry*y;
cnr = cnry*y;
k = ky*y;
iv = delt*k;
l = ly*y;
c = cy*y;
z = str*y;
b = sb*y;
gc = sgc*y;
gi = sgi*y;
kg = gi/deltg;
lambr = ((1-theta)^(-gamm))*(cr^(-gamm));
lambnr = ((1-theta)^(-gamm))*(cnr^(-gamm));


ss = [w k l c y phi R rk psi1 cr cnr iv b gc gi kg lambr lambnr ky ly cy str z];