% Estimate the posterior modes for the New Keysian model in "Foresight and
% Information Flows" by Eric M. Leeper, Todd B. Walker, and Shu-Chun S.
% Yang

clear all
close all

MH=0;

rand('state',26);
randn('state',26);

nendog = 33;    % number of endogenous variables
nexog = 10;     % number of exogenous variables
nforerrors = 9;
nvar = nendog + nexog;
nparam = 34;

% Define variables from DSGE model
variables;

% Pick observables
obs = zeros(nexog,1);
obs(1,1) = Ni;
obs(2,1) = Nc;
obs(3,1) = NR;
obs(4,1) = NTl;
obs(5,1) = NTk;
obs(6,1) = Ng;
obs(7,1) = Nz;
obs(8,1) = Nl;
obs(9,1) = Npi;
obs(10,1) = Nw;

%Get data
[obsdata,H] = data_detrend(obs,nvar);
 save logpostinfo.mat obs obsdata

% ====================================
% ======Find Posterior Modes ========
% ====================================
% Solve using Chris Sims' optimization program, csminwel
Hinit = 1e-4*eye(nparam);
num = 10; % number of posterior mode searches from different starting points
[xhat,Hhat,fhat,params,retcode] = findmode(Hinit,num,nparam,nvar,nexog,nforerrors);
[opt] = optmode(fhat,Hhat,retcode);

save modes.mat xhat Hhat fhat params retcode opt
    
