 function value = kalman2(Y,F,Q,P,X,T,obs,obspos)
% function value = kalman(Y,F,Q,P,X,T,obs,obspos)
% This calculates one-step ahead predictions using the Kalman filter for the case when 
% the A and R matrices in Hamilton are zero.
% OUTPUT:
% value: log likelihood value
% Last Updated: August 14, 2009 by Nora Traum

    n = size(F,2);

    t=0;
    lik  = zeros(T+1,1);
    
    while t < T
        t  = t+1;
        v = Y(:,t) - X(obspos);
        MSE  = P(obspos,obspos); 
        oldP  = P;
        if det(MSE) <= 0
            value = -Inf;
        return
        else
            detMSE = det(MSE);
        end
        invMSE = inv(MSE);
        lik(t) = log(detMSE) + transpose(v)*invMSE*v;
        K = P(:,obspos)*invMSE; %% premultiplication by the transition matrix T is removed (stephane)
        X = F*(X + K*v);		
        P = F*(P - K*P(obspos,:))*transpose(F) + Q;	
    end
    
    value = -0.5*sum(lik(1:end)) - ((obs*T)/2)*log(2*pi);
    % If value is NaN, change it to negative infinity.  This must be done or
    % the optimization of the posterior distribution will get stuck at an NaN
    % value.
    if (isnan(value)) == 1
    value = -Inf;
    end
    

