% Compare actual and estimated output multipliers by identified VAR using the RBC model
% in "Foresight and % Information Flows" by Eric M. Leeper, Todd B. Walker, and Shu-Chun S. Yang

%====================================================================
%======== Quantative Importance: four information process ===========
%====================================================================

close all
clear all
load param;   
variables;

cor = .261;            %\xi in the paper
infoprocess =1;        %Specify information flow process: I~IV


withtauk = 0;           %1: estimate a system with tauk; 0: estimate a system with taul; 
if withtauk ==1;
    obs= [Ntauk;Ni;Ny;Nrev];
%    obs= [Ntauk;Nrev;Ny;Nc];
%     obs= [Ntauk;Nrev;Ny;Nl];
%    obs= [Ntauk;Ny;Nl;Nrev];
else
    obs= [Ntaul;Ni;Ny;Nrev];%;Nb];
%    obs= [Ntaul;Nrev;Ny;Nc];
%    obs= [Ntaul;Nrev;Ny;Nl];
%     obs= [Ntaul;Ny;Nl;Nrev];
end


if infoprocess == 1;  %legislative lags: 6 quarters with smooth MA coefficients
    ma0=0.0; ma1=1/6; ma2=1/6; ma3=1/6; ma4=1/6; ma5=1/6; ma6=1/6; ma7=0; 
    ma8=1-ma0-ma1-ma2-ma3-ma4-ma5-ma6-ma7; %0
elseif infoprocess == 2;  %Legislative lags: 6 quarters with large weights in quarters 4-6
    ma0=0.0; ma1=0.05; ma2=0.05; ma3=0.05; ma4=0.25; ma5=0.3; ma6=0.3; ma7=0; 
    ma8=1-ma0-ma1-ma2-ma3-ma4-ma5-ma6-ma7; %0 
elseif infoprocess == 3;  %Implementation lags: 8 quarter phase in; perfect foresight
    ma0=0.0; ma1=0.0; ma2=0.0; ma3=0.0; ma4=0.0; ma5=0.0; ma6=0; ma7=0; 
    ma8=1-ma0-ma1-ma2-ma3-ma4-ma5-ma6-ma7; %1 
elseif  infoprocess == 4;  %inforprocess ==4; Implementation lags: 2 quarter phase in; perfect foresight
    ma0=0.0; ma1=0.0; ma2=1.0; ma3=0.0; ma4=0.0; ma5=0.0; ma6=0; ma7=0; ma8=0;
else % no foresight
  ma0=1; ma1=0.0; ma2=.0; ma3=0.0; ma4=0.0; ma5=0.0; ma6=0; ma7=0; ma8=0;
end



nendog=29;%31;
nexog= 4;%5;
nforerrors=2;
nvar = nendog+nexog;
VARn = nexog; 
steps=61;               % length of impulse responses

variables; 

[G0,G1,Psi,Pi,CC,ss] = modelnogrowth(param,nvar,nforerrors,ma1,ma2,ma3,ma4,ma5,ma6,ma7,ma8,cor);

ssnames ={'css';'iss';'yss';'lss';'tkss';'tlss';'gss';'revss';'kss';'R'};
for j = 1:length(ss)
   eval([char(ssnames(j,:)) ' = ss(j);']);   
end
[G,C0,M,fmat,fwt,ywt,gev,eu] = gensys(G0,G1,CC,Psi,Pi);
disp(['eu:    ' mat2str(eu)])
%disp(['invertibility condition satisfied (1--yes):    ' mat2str(invtidx)])

%===============================================================
%===============================================================
%=============== Obtain theoretical IR ========================= 
%===============================================================
%===============================================================

omega = eye(nexog);
omegafill = zeros(nvar,nvar);
omegafill(nvar-nexog+1:nvar,nvar-nexog+1:nvar)=omega;

Bh = G';
swish = M*omegafill;  % covariance matrix
lags =1;
nn = [nvar lags steps];
imf = impulse0(Bh,swish,nn);

time =0:steps-1;% (0:steps-1)/4;   %convert it to years


%===============================================================
%== BP Multiplier: to a $1 maximum increase in revenues ========
%===============================================================
%tauk shock
y_utk(:,1) = (imf(:,nvar*(Nutk-1)+Ny)*yss);
rev_utk(:,1) = (imf(:,nvar*(Nutk-1)+Nrev)*revss);
mY_utk(:,1) = y_utk(:,1)/max(imf(:,nvar*(Nutk-1)+Nrev)*revss);
mREV_utk(:,1) = rev_utk(:,1)/max(imf(:,nvar*(Nutk-1)+Nrev)*revss);
peakmY_utk_idx = find(mY_utk == min(mY_utk));

%taul shock
y_utl(:,1) = (imf(:,nvar*(Nutl-1)+Ny)*yss);
rev_utl(:,1) = (imf(:,nvar*(Nutl-1)+Nrev)*revss);
mY_utl(:,1) = y_utl(:,1)/max(imf(:,nvar*(Nutl-1)+Nrev)*revss);
mREV_utl(:,1) = rev_utl(:,1)/max(imf(:,nvar*(Nutl-1)+Nrev)*revss);
peakmY_utl_idx = find(mY_utl == min(mY_utl));

if withtauk==1
    disp('  ')
    disp('True Output Multipliers--Capital Tax Cut')
    disp(['0Q:    ' num2str(mY_utk(1,1))])
    disp(['1Q:    ' num2str(mY_utk(2,1))])
    disp(['4Q:    ' num2str(mY_utk(5,1))])
    disp(['8Q:    ' num2str(mY_utk(9,1))])
    disp(['12Q:    ' num2str(mY_utk(13,1))])
    disp(['20Q:    ' num2str(mY_utk(21,1))])
    disp(['peak:   ' num2str(min(mY_utk))])
    disp(['peak quarter:   ' num2str(peakmY_utk_idx-1)])

else
    disp('  ')
    disp('True Output Multipliers--Labor Tax Cut')
    disp(['0Q:    ' num2str(mY_utl(1,1))])
    disp(['1Q:    ' num2str(mY_utl(2,1))])
    disp(['4Q:    ' num2str(mY_utl(5,1))])
    disp(['8Q:    ' num2str(mY_utl(9,1))])
    disp(['12Q:    ' num2str(mY_utl(13,1))])
    disp(['20Q:    ' num2str(mY_utl(21,1))])
    disp(['peak:   ' num2str(min(mY_utl))])
    disp(['peak quarter:   ' num2str(peakmY_utl_idx-1)])
end

%===============================================================
%===============================================================
%=============== Obtain econometrician IR ====================== 
%===============================================================
%===============================================================
M=M(:,nendog+1:end);
 [invtidx,eigvalue, modulus, matrices,A,B,C,D] = invertest_abcd(G,C0,M,nexog,0,obs);
disp(['invertibility condition (if 0, not satisfied):    ' mat2str(invtidx)])

%A11=[param(1,24),param(1,30),param(1,31),param(1,32),param(1,33)]; %sigma's for shocks in the order in variables.
A11=[param(1,24),param(1,30),param(1,32),param(1,33)]; %sigma's for shocks in the order in variables.
A0=zeros(nexog);
A0=diag(A11);  
[IRFa, IRFe,bbb,ddd]=ABCD_Kfilter(A,B,C,D,A0,steps); %bbb is nvar X VARn
     
if withtauk==1
    i=1;%find(obs==Ntauk);
    IRFeTauK(:,:)=IRFe(:,:,i);
    % % Code Check for K-filter
    % % [1] Does agent's response correspond with theoretical model
    IRFaTauK(:,:)=IRFa(:,:,Nutk-nendog);
    j=find(obs==Ny);
    if  abs(sum(IRFaTauK(:,j)-imf(:,nvar*(Nutk-1)+Ny)))>0.1 % output
        display('Discard Results: IRFs do not allign')
    end
        
    h=find(obs==Nrev);    
    estY_utk(:,1) = IRFeTauK(:,j)*yss;
    estREV_utk(:,1) = IRFeTauK(:,h)*revss;
    estmY_utk(:,1) = estY_utk(:,1)/max(IRFeTauK(:,h)*revss);
    estmREV_utk(:,1) = estREV_utk(:,1)/max(IRFeTauK(:,h)*revss);
    peakestmY_utk_idx = find(estmY_utk == min(estmY_utk));             
else
   i=1;%find(obs==Ntaul);
   IRFeTauL(:,:)=IRFe(:,:,i);
   % % Code Check for K-filter
   % % [1] Does agent's response correspond with theoretical model
   IRFaTauL(:,:)=IRFa(:,:,Nutl-nendog);
   j=find(obs==Ny);
   if  sum(IRFaTauL(:,j)-imf(:,nvar*(Nutl-1)+Ny))>0.1 % output
        display('Discard Results: IRFs do not allign')
   end
   h=find(obs==Nrev);
   estY_utl(:,1) = IRFeTauL(:,j)*yss;
   estREV_utl(:,1) = IRFeTauL(:,h)*revss;
   estmY_utl(:,1) = estY_utl(:,1)/max(IRFeTauL(:,h)*revss);
   estmREV_utl(:,1) = estREV_utl(:,1)/max(IRFeTauL(:,h)*revss);
   peakestmY_utl_idx = find(estmY_utl == min(estmY_utl));
end

if withtauk==1
    disp('  ')
    disp('Estimated Output Multipliers--Capital Tax Cut')
    disp(['0Q:    ' num2str(estmY_utk(1,1))])
    disp(['1Q:    ' num2str(estmY_utk(2,1))])
    disp(['4Q:    ' num2str(estmY_utk(5,1))])
    disp(['8Q:    ' num2str(estmY_utk(9,1))])
    disp(['12Q:    ' num2str(estmY_utk(13,1))])
    disp(['20Q:    ' num2str(estmY_utk(21,1))])
    disp(['peak:   ' num2str(min(estmY_utk))])
    disp(['peak quarter:   ' num2str(peakestmY_utk_idx-1)])
else
    disp('  ')
    disp('Estimated Output Multipliers--Labor Tax Cut')
    disp(['0Q:    ' num2str(estmY_utl(1,1))])
    disp(['1Q:    ' num2str(estmY_utl(2,1))])
    disp(['4Q:    ' num2str(estmY_utl(5,1))])
    disp(['8Q:    ' num2str(estmY_utl(9,1))])
    disp(['12Q:    ' num2str(estmY_utl(13,1))])
    disp(['20Q:    ' num2str(estmY_utl(21,1))])
    disp(['peak:   ' num2str(min(estmY_utl))])
    disp(['peak quarter:   ' num2str(peakestmY_utl_idx-1)])
end

%===============================================================
%=============== Compute bias ====================== 
%===============================================================
if withtauk==1
    biasY_utk = estmY_utk-mY_utk;
    biasYper_utk = 100*((estmY_utk-mY_utk)./abs(mY_utk));

    disp('  ')
    disp('Bias in $--Capital Tax Cut')
    disp(['0Q:    ' num2str(biasY_utk(1,1))])
    disp(['1Q:    ' num2str(biasY_utk(2,1))])
    disp(['4Q:    ' num2str(biasY_utk(5,1))])
    disp(['8Q:    ' num2str(biasY_utk(9,1))])
    disp(['12Q:    ' num2str(biasY_utk(13,1))])
    disp(['20Q:    ' num2str(biasY_utk(21,1))])
   
    disp('  ')
    disp('Bias in %--Capital Tax Cut')
    disp(['0Q:    ' num2str(biasYper_utk(1,1))])
    disp(['1Q:    ' num2str(biasYper_utk(2,1))])
    disp(['4Q:    ' num2str(biasYper_utk(5,1))])
    disp(['8Q:    ' num2str(biasYper_utk(9,1))])
    disp(['12Q:    ' num2str(biasYper_utk(13,1))])
    disp(['20Q:    ' num2str(biasYper_utk(21,1))])

else
    biasY_utl = estmY_utl-mY_utl;
    biasYper_utl = 100*((estmY_utl-mY_utl)./abs(mY_utl));
    disp('  ')
    disp('Bias in $--Labor Tax Cut')
    disp(['0Q:    ' num2str(biasY_utl(1,1))])
    disp(['1Q:    ' num2str(biasY_utl(2,1))])
    disp(['4Q:    ' num2str(biasY_utl(5,1))])
    disp(['8Q:    ' num2str(biasY_utl(9,1))])
    disp(['12Q:    ' num2str(biasY_utl(13,1))])
    disp(['20Q:    ' num2str(biasY_utl(21,1))])
    disp('  ')
    disp('Bias in %--Labor Tax Cut')
    disp(['0Q:    ' num2str(biasYper_utl(1,1))])
    disp(['1Q:    ' num2str(biasYper_utl(2,1))])
    disp(['4Q:    ' num2str(biasYper_utl(5,1))])
    disp(['8Q:    ' num2str(biasYper_utl(9,1))])
    disp(['12Q:    ' num2str(biasYper_utl(13,1))])
    disp(['20Q:    ' num2str(biasYper_utl(21,1))])

end


